/*
 * assert.c - Debug assertion support.
 *
 * This file is part of the Portable.NET C library.
 * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

void
__assert_fail (const char *assertion, const char *file,
               unsigned int line, const char *function)
{
  fprintf (stderr, "Assertion %s failed in %s (%s, line %u)\n",
  	   assertion, function, file, line);
  exit (3);
}

void
__assert(const char *assertion, const char *file, unsigned int line)
{
  fprintf (stderr, "Assertion %s failed (%s, line %u)\n",
  	   assertion, file, line);
  exit (3);
}
