/*
 * <assembly/Commons.Xml.Relaxng.h> - Import all of the
 *           definitions in "Commons.Xml.Relaxng.dll".
 *
 * This file is part of the Portable.NET C library.
 * Copyright (C) 2004  Southern Storm Software, Pty Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _ASSEMBLY_COMMONS_XML_RELAXNG_H
#define _ASSEMBLY_COMMONS_XML_RELAXNG_H

#include <csharp.h>

__BEGIN_DECLS

#using <Commons.Xml.Relaxng.h>

using namespace Commons::Xml;
using namespace Commons::Xml::Relaxng;
using namespace Commons::Xml::Relaxng::Derivative;

__END_DECLS

#endif  /* !_ASSEMBLY_COMMONS_XML_RELAXNG_H */
